package middleware

import (
	"fmt"
	"net/http"

	"hello-api/internal/logger"

	"hello-api/internal/app/parsers/devicecontext"
	"hello-api/internal/utils"

	"github.com/gin-gonic/gin"
)

func ValidateDeviceContext(context *gin.Context) {
	deviceAuthContext, err := utils.ContextGet[*devicecontext.DeviceContext](context, DeviceContextHeader)
	if err != nil {
		_ = context.AbortWithError(http.StatusInternalServerError, DeviceContextNotFoundError)
		return
	}

	validationErr := deviceAuthContext.Validate()
	if validationErr != nil {
		_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("Device Context Validation Middleware didn't recieve valid device context %w", validationErr))
		return
	}
	logger.BaseLogger().Debug(fmt.Sprintf("Validated Device context -- %+v", deviceAuthContext))
	context.Next()
}
