package middleware

import (
	"net/http"
	"net/http/httptest"

	"hello-api/internal/app/parsers/devicecontext"

	deviceauthpb "carbonblack.com/protocols/psc/ems/server/deviceauth"
	"github.com/gin-gonic/gin"
	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
)

var _ = Describe("Device context validation middleware", func() {
	It("returns HTTP 500 when the context header is not supplied", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		ValidateDeviceContext(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusInternalServerError))
	})
	It("returns HTTP 500 when the context header has a nil device", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		testContext.Set(DeviceContextHeader, &devicecontext.DeviceContext{Context: deviceauthpb.Context{Device: nil}})
		ValidateDeviceContext(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusInternalServerError))
	})
	It("returns HTTP 500 when the context header has a nil group", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		testContext.Set(DeviceContextHeader, &devicecontext.DeviceContext{Context: deviceauthpb.Context{Device: new(deviceauthpb.Device), DeviceGroup: nil}})
		ValidateDeviceContext(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusInternalServerError))
	})
	It("returns HTTP 500 when the context header has a nil org", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		testContext.Set(DeviceContextHeader, &devicecontext.DeviceContext{Context: deviceauthpb.Context{Device: new(deviceauthpb.Device), DeviceGroup: &deviceauthpb.DeviceGroup{}, Organization: nil}})
		ValidateDeviceContext(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusInternalServerError))
	})
	It("returns HTTP 200 when the context header is valid", func() {
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		testContext.Set(DeviceContextHeader, &devicecontext.DeviceContext{Context: deviceauthpb.Context{Device: new(deviceauthpb.Device), DeviceGroup: &deviceauthpb.DeviceGroup{}, Organization: &deviceauthpb.Organization{}}})
		ValidateDeviceContext(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusOK))
	})
})
